using System;
using System.IO;
using System.Net;
using System.Net.Sockets;
using System.Web;
using System.Web.UI;
using System.Collections;


namespace com.inet.helpdesk {

    public class Proxy : Page {

        private String host = "LocalHost";
        private String port = "8000";
        private String port2 = "8002";

        private HttpRequest Request;
        private HttpResponse Response;
        private HttpApplicationState Application;

        private const int MAX_SOCKETS = 10;

        public virtual void ProcessRequest(HttpContext context) {
            Request = context.Request;
            Response = context.Response;
            Application = context.Application;

            String url = Request.QueryString["url"];
            if (url != null) {
                url = Server.UrlDecode(url);
                String http = "GET " + url + "?" + Request.QueryString + " HTTP/1.0\r\nHost: " + Request.UserHostName + "\r\n\r\n";
                System.Text.Encoding encoding = System.Text.Encoding.GetEncoding("ISO8859-1");
                HttpWebRequest hdRequest = (HttpWebRequest)WebRequest.Create("http://" + host + ":" + port2 + url + "?" + Request.QueryString);

                // Transfer Headers
                HttpWebResponse hdResponse;
                try {
                    hdResponse = (HttpWebResponse)hdRequest.GetResponse();
                } catch (WebException ex) {
                    hdResponse = (HttpWebResponse)ex.Response;
                }
                Response.ClearHeaders();
                Response.StatusCode = (int)hdResponse.StatusCode;
                Response.StatusDescription = hdResponse.StatusDescription;
                foreach (String key in hdResponse.Headers.Keys) {
                    if (String.Compare(key, "transfer-encoding", true) != 0 && String.Compare(key, "server", true) != 0) {
                        String value = hdResponse.Headers[key];
                        Response.AddHeader(key, value);
                    }
                }

                // Transfer Data in a loop
                Stream input = hdResponse.GetResponseStream();
                Stream output = Response.OutputStream;
                byte[] bytes = new byte[4000];
                while (true) {
                    int n = input.Read(bytes, 0, 4000);
                    if (n <= 0) {
                        break;
                    }
                    output.Write(bytes, 0, n);
                }
                input.Close();
                output.Close();
            } else {
                Stream input = Request.InputStream;
                if (Request.ContentLength == 0) {
                    Response.StatusCode = 500;
                    Response.Write("ContentLength is 0.");
                    return;
                }
                TcpClient socket = getSocket();
                Stream output = socket.GetStream();
                transferData(input, output, Request);
                output = Response.OutputStream;
                input = socket.GetStream();
                transferData(input, output, Request);

                putSocket(socket);
            }
        }

        private TcpClient getSocket() {
            lock (Application) {
                ArrayList list = (ArrayList)Application["TcpClient"];
                if (list == null) {
                    Application["TcpClient"] = list = new ArrayList();
                }
                while (list.Count > 0) {
                    TcpClient socket = (TcpClient)list[0];
                    list.RemoveAt(0);
                    if (socket.Client.Connected && socket.Available == 0 ) {
                        return socket;
                    }
                }
            }
            return new TcpClient(host, Int32.Parse(port));
        }

        private void putSocket(TcpClient socket) {
            lock (Application) {
                ArrayList list = (ArrayList)Application["TcpClient"];
                if (list.Count < MAX_SOCKETS) {
                    list.Add(socket);
                } else {
                    socket.Close();
                }
            }
        }

        private void transferData(Stream input, Stream output, HttpRequest req) {
            byte[] puffer = new byte[4096];
            int count;
            bool lastPacket = false;
            bool isPacketStart = true;
            bool lowByteRead = false;
            bool isLogin = false;
            byte lo = 0, hi = 0;
            int packetSize = 0, sizeRead = 0;
            do {
                count = input.Read(puffer, 0, puffer.Length);
                if (count > 0) {
                    output.Write(puffer, 0, count);
                    sizeRead += count;
                    int nextPacketIdx = 0;

                    while (isPacketStart || sizeRead > packetSize) {
                        if (isPacketStart) {
                            if (!lowByteRead && sizeRead == 1) {
                                lo = puffer[nextPacketIdx];
                                lowByteRead = true;
                                break;
                            }
                            if (lowByteRead) {
                                hi = puffer[nextPacketIdx];
                                lowByteRead = false;
                            } else {
                                lo = puffer[nextPacketIdx];
                                hi = puffer[nextPacketIdx + 1];
                            }
                            packetSize = (lo & 0xFF) + (hi << 8);
                            lastPacket = (packetSize & 0x8000) == 0;
                            isLogin = (packetSize & 0x4000) != 0;
                            packetSize &= 0x3fff;
                            isPacketStart = false;
                            continue;
                        }
                        nextPacketIdx = packetSize - (sizeRead - count);
                        sizeRead -= packetSize;
                        isPacketStart = true;
                        if (sizeRead == 0) {
                            packetSize = 0;
                            break;
                        }
                    }
                    if (sizeRead == packetSize) {
                        sizeRead = 0;
                        packetSize = 0;
                        isPacketStart = true;
                    }
                } else break;
            } while (!lastPacket || (sizeRead != packetSize));
            output.Flush();
            if (isLogin) {
                String host = req.UserHostName;
                if (host == null) host = "unknown host";
                int length = host.Length;
                int idx = 2;
                puffer[idx++] = (byte)length;
                puffer[idx++] = (byte)(length >> 8);
                puffer[idx++] = (byte)(length >> 16);
                puffer[idx++] = (byte)(length >> 24);
                for (int i = 0; i < length; i++) {
                    int c = host[i];
                    puffer[idx++] = (byte)c;
                    puffer[idx++] = (byte)(c >> 8);
                }
                length = idx;
                idx = 0;
                puffer[idx++] = (byte)length;
                puffer[idx++] = (byte)(length >> 8);
                output.Write(puffer, 0, length);
                output.Flush();
            }

        }

    }
}